/*
 * [EntifyStrings.java]
 *
 * Summary: Inserts HTML entities such as &quot; into a String replacing the single character equivalents.
 *
 * Copyright: (c) 2005-2011 Roedy Green, Canadian Mind Products, http://mindprod.com
 *
 * Licence: This software may be copied and used freely for any purpose but military.
 *          http://mindprod.com/contact/nonmil.html
 *
 * Requires: JDK 1.5+
 *
 * Created with: JetBrains IntelliJ IDEA IDE http://www.jetbrains.com/idea/
 *
 * Version History:
 *  2.6 2009-04-05 StripEntities now leaves a space behind when it removes a <br /><p><td> etc tag.
 *  2.7 2009-11-14 generate a table for the HTML cheat sheet of quote-like entities.
 *  2.8 2009-12-22 export table on HTML 5 entities. Now import csv file rather than embed entity facts.
 *  2.9 2010-01-29 export XHTML entities (currently same as HTML-4 entities).
 *  3.0 2011-01-05 remove deprecated methods. add toHTMLEntity, toXMLEntity, hide charToHTMLEntity, charToXMLEntity
 *  3.1 2011-02-10 rename methods to htmlEntify and xmlEntify
 */
package com.mindprod.entities;

/**
 * Inserts HTML entities such as &quot; into a String replacing the single character equivalents.
 * <p/>
 * Entities are coded inline in a giant case.
 *
 * @author Roedy Green, Canadian Mind Products
 * @version 3.1 2011-02-10 rename methods to entifyHTML and entifyXML
 * @see DeEntify
 * @see DeEntifyStrings
 * @see Entify
 * @see EntifyStrings
 * @see Flatten
 * @since 2005
 */
public class EntifyStrings
    {
    // -------------------------- PUBLIC STATIC METHODS --------------------------

    /**
     * Converts text to HTML by quoting dangerous characters. Text must not already contain entities. e.g. " ==> &quot;
     * < ==> &lt; ordinary text passes unchanged. Does not convert space to &nbsp;
     *
     * @param text raw text to be processed. Must not be null.
     *
     * @return translated HTML text, or null if input is null.
     * @noinspection WeakerAccess
     */
    public static String entifyHTML( String text )
        {
        return entifyHTMLorXML( text, false );
        }

    /**
     * Converts text to XML by quoting dangerous characters.
     * Text must not already contain entities. e.g. " ==> &quot;
     * < ==> &lt; ordinary text passes unchanged. Does not convert space to &nbsp;
     *
     * @param text raw text to be processed. Must not be null.
     *
     * @return translated XML text, or null if input is null.
     * @noinspection WeakerAccess
     */
    public static String entifyXML( String text )
        {
        return entifyHTMLorXML( text, true );
        }

    /**
     * Converts text to HTML by quoting dangerous characters.;
     * Does not convert space to &nbsp;
     *
     * @param c raw character.
     *
     * @return translated HTML text, eg. & -> &amp  x -> x
     * @noinspection WeakerAccess
     */
    public static String toHTMLEntity( char c )
        {
        final String result = charToHTMLEntity( c );
        return result == null ? String.valueOf( c ) : result;
        }

    /**
     * Converts text to XML by quoting dangerous characters.;
     * Does not convert space to &nbsp;
     *
     * @param c raw character.
     *
     * @return translated HTML text, eg. < -> &lt;  x -> x
     * @noinspection WeakerAccess
     */
    public static String toXMLEntity( char c )
        {
        final String result = charToXMLEntity( c );
        return result == null ? String.valueOf( c ) : result;
        }

    // -------------------------- STATIC METHODS --------------------------

    /**
     * convert a single char to its equivalent HTML entity. Ordinary chars are not changed. 160 -> &nbsp;  weird chars
     * -> &#nnn; form
     *
     * @param c Char to convert
     *
     * @return equivalent string e.g. &amp;, null means leave char as is.
     */
    private static String charToHTMLEntity( char c )
        {
        switch ( c )
            {
            default:
                if ( c < 127 )
                    {
                    // leave alone as equivalent string.
                    return null;
                    // faster than String.valueOf( c ).intern();
                    }
                else
                    {
                    // use the &#nnn; form
                    return "&#" + Integer.toString( c ) + ";";
                    }
                // W A R N I N G !  _ M A N U A L L Y  _ I N S E R T E D _ G E N E R A T E D  _ C O D E
                // this code generated by Entities, include from com\mindprod\entities\entitiescase.javafrag
                // Could be more efficiently handled with a lookup table[9831]
            case 34:
                return "&quot;"     /* &#x0022; quotation mark */;
            case 38:
                return "&amp;"      /* &#x0026; ampersand */;
            case 60:
                return "&lt;"       /* &#x003c; less-than sign */;
            case 62:
                return "&gt;"       /* &#x003e; greater-than sign */;
            case 160:
                return "&nbsp;"     /* &#x00a0; non-breaking space */;
            case 161:
                return "&iexcl;"    /* &#x00a1; inverted exclamation mark */;
            case 162:
                return "&cent;"     /* &#x00a2; cent sign */;
            case 163:
                return "&pound;"    /* &#x00a3; pound sign */;
            case 164:
                return "&curren;"   /* &#x00a4; currency sign */;
            case 165:
                return "&yen;"      /* &#x00a5; yen sign */;
            case 166:
                return "&brvbar;"   /* &#x00a6; broken bar */;
            case 167:
                return "&sect;"     /* &#x00a7; section sign */;
            case 168:
                return "&uml;"      /* &#x00a8; diaeresis */;
            case 169:
                return "&copy;"     /* &#x00a9; copyright sign circled c */;
            case 170:
                return "&ordf;"     /* &#x00aa; feminine ordinal indicator */;
            case 171:
                return "&laquo;"    /* &#x00ab; left guillemot */;
            case 172:
                return "&not;"      /* &#x00ac; not sign */;
            case 173:
                return "&shy;"      /* &#x00ad; soft hyphen */;
            case 174:
                return "&reg;"      /* &#x00ae; registered sign. circled R. */;
            case 175:
                return "&macr;"     /* &#x00af; macron */;
            case 176:
                return "&deg;"      /* &#x00b0; degree sign */;
            case 177:
                return "&plusmn;"   /* &#x00b1; plus-minus sign */;
            case 178:
                return "&sup2;"     /* &#x00b2; superscript two */;
            case 179:
                return "&sup3;"     /* &#x00b3; superscript three */;
            case 180:
                return "&acute;"    /* &#x00b4; acute accent */;
            case 181:
                return "&micro;"    /* &#x00b5; micro sign */;
            case 182:
                return "&para;"     /* &#x00b6; pilcrow sign */;
            case 183:
                return "&middot;"   /* &#x00b7; middle dot */;
            case 184:
                return "&cedil;"    /* &#x00b8; cedilla */;
            case 185:
                return "&sup1;"     /* &#x00b9; superscript one */;
            case 186:
                return "&ordm;"     /* &#x00ba; masculine ordinal indicator */;
            case 187:
                return "&raquo;"    /* &#x00bb; right guillemot */;
            case 188:
                return "&frac14;"   /* &#x00bc; vulgar fraction 1/4 */;
            case 189:
                return "&frac12;"   /* &#x00bd; vulgar fraction 1/2 */;
            case 190:
                return "&frac34;"   /* &#x00be; vulgar fraction 3/4 */;
            case 191:
                return "&iquest;"   /* &#x00bf; inverted question mark */;
            case 192:
                return "&Agrave;"   /* &#x00c0; Latin capital letter A with grave */;
            case 193:
                return "&Aacute;"   /* &#x00c1; Latin capital letter A with acute */;
            case 194:
                return "&Acirc;"    /* &#x00c2; Latin capital letter A with circumflex */;
            case 195:
                return "&Atilde;"   /* &#x00c3; Latin capital letter A with tilde */;
            case 196:
                return "&Auml;"     /* &#x00c4; Latin capital letter A with diaeresis */;
            case 197:
                return "&Aring;"    /* &#x00c5; Latin capital letter A with ring above */;
            case 198:
                return "&AElig;"    /* &#x00c6; Latin capital letter AE */;
            case 199:
                return "&Ccedil;"   /* &#x00c7; Latin capital letter C with cedilla */;
            case 200:
                return "&Egrave;"   /* &#x00c8; Latin capital letter E with grave */;
            case 201:
                return "&Eacute;"   /* &#x00c9; Latin capital letter E with acute */;
            case 202:
                return "&Ecirc;"    /* &#x00ca; Latin capital letter E with circumflex */;
            case 203:
                return "&Euml;"     /* &#x00cb; Latin capital letter E with diaeresis */;
            case 204:
                return "&Igrave;"   /* &#x00cc; Latin capital letter I with grave */;
            case 205:
                return "&Iacute;"   /* &#x00cd; Latin capital letter I with acute */;
            case 206:
                return "&Icirc;"    /* &#x00ce; Latin capital letter I with circumflex */;
            case 207:
                return "&Iuml;"     /* &#x00cf; Latin capital letter I with diaeresis */;
            case 208:
                return "&ETH;"      /* &#x00d0; Latin capital letter Eth */;
            case 209:
                return "&Ntilde;"   /* &#x00d1; Latin capital letter N with tilde */;
            case 210:
                return "&Ograve;"   /* &#x00d2; Latin capital letter O with grave */;
            case 211:
                return "&Oacute;"   /* &#x00d3; Latin capital letter O with acute */;
            case 212:
                return "&Ocirc;"    /* &#x00d4; Latin capital letter O with circumflex */;
            case 213:
                return "&Otilde;"   /* &#x00d5; Latin capital letter O with tilde */;
            case 214:
                return "&Ouml;"     /* &#x00d6; Latin capital letter O with diaeresis */;
            case 215:
                return "&times;"    /* &#x00d7; multiplication sign */;
            case 216:
                return "&Oslash;"   /* &#x00d8; Latin capital letter O with stroke */;
            case 217:
                return "&Ugrave;"   /* &#x00d9; Latin capital letter U with grave */;
            case 218:
                return "&Uacute;"   /* &#x00da; Latin capital letter U with acute */;
            case 219:
                return "&Ucirc;"    /* &#x00db; Latin capital letter U with circumflex */;
            case 220:
                return "&Uuml;"     /* &#x00dc; Latin capital letter U with diaeresis */;
            case 221:
                return "&Yacute;"   /* &#x00dd; Latin capital letter Y with acute */;
            case 222:
                return "&THORN;"    /* &#x00de; Latin capital letter Thorn */;
            case 223:
                return "&szlig;"    /* &#x00df; Latin small letter sharp s */;
            case 224:
                return "&agrave;"   /* &#x00e0; Latin small letter a with grave */;
            case 225:
                return "&aacute;"   /* &#x00e1; Latin small letter a with acute */;
            case 226:
                return "&acirc;"    /* &#x00e2; Latin small letter a with circumflex */;
            case 227:
                return "&atilde;"   /* &#x00e3; Latin small letter a with tilde */;
            case 228:
                return "&auml;"     /* &#x00e4; Latin small letter a with diaeresis */;
            case 229:
                return "&aring;"    /* &#x00e5; Latin small letter a with ring above */;
            case 230:
                return "&aelig;"    /* &#x00e6; Latin lowercase ligature ae */;
            case 231:
                return "&ccedil;"   /* &#x00e7; Latin small letter c with cedilla */;
            case 232:
                return "&egrave;"   /* &#x00e8; Latin small letter e with grave */;
            case 233:
                return "&eacute;"   /* &#x00e9; Latin small letter e with acute */;
            case 234:
                return "&ecirc;"    /* &#x00ea; Latin small letter e with circumflex */;
            case 235:
                return "&euml;"     /* &#x00eb; Latin small letter e with diaeresis */;
            case 236:
                return "&igrave;"   /* &#x00ec; Latin small letter i with grave */;
            case 237:
                return "&iacute;"   /* &#x00ed; Latin small letter i with acute */;
            case 238:
                return "&icirc;"    /* &#x00ee; Latin small letter i with circumflex */;
            case 239:
                return "&iuml;"     /* &#x00ef; Latin small letter i with diaeresis */;
            case 240:
                return "&eth;"      /* &#x00f0; Latin small letter eth */;
            case 241:
                return "&ntilde;"   /* &#x00f1; Latin small letter n with tilde */;
            case 242:
                return "&ograve;"   /* &#x00f2; Latin small letter o with grave */;
            case 243:
                return "&oacute;"   /* &#x00f3; Latin small letter o with acute */;
            case 244:
                return "&ocirc;"    /* &#x00f4; Latin small letter o with circumflex */;
            case 245:
                return "&otilde;"   /* &#x00f5; Latin small letter o with tilde */;
            case 246:
                return "&ouml;"     /* &#x00f6; Latin small letter o with diaeresis */;
            case 247:
                return "&divide;"   /* &#x00f7; division sign */;
            case 248:
                return "&oslash;"   /* &#x00f8; Latin small letter o with stroke */;
            case 249:
                return "&ugrave;"   /* &#x00f9; Latin small letter u with grave */;
            case 250:
                return "&uacute;"   /* &#x00fa; Latin small letter u with acute */;
            case 251:
                return "&ucirc;"    /* &#x00fb; Latin small letter u with circumflex */;
            case 252:
                return "&uuml;"     /* &#x00fc; Latin small letter u with diaeresis */;
            case 253:
                return "&yacute;"   /* &#x00fd; Latin small letter y with acute */;
            case 254:
                return "&thorn;"    /* &#x00fe; Latin small letter thorn */;
            case 255:
                return "&yuml;"     /* &#x00ff; Latin small letter y with diaeresis */;
            case 338:
                return "&OElig;"    /* &#x0152; Latin capital ligature oe */;
            case 339:
                return "&oelig;"    /* &#x0153; Latin small ligature oe */;
            case 352:
                return "&Scaron;"   /* &#x0160; Latin capital letter S with caron */;
            case 353:
                return "&scaron;"   /* &#x0161; Latin small letter s with caron */;
            case 376:
                return "&Yuml;"     /* &#x0178; Latin capital letter Y with diaeresis */;
            case 402:
                return "&fnof;"     /* &#x0192; Latin small letter f with hook */;
            case 710:
                return "&circ;"     /* &#x02c6; modifier letter circumflex accent */;
            case 732:
                return "&tilde;"    /* &#x02dc; small tilde */;
            case 913:
                return "&Alpha;"    /* &#x0391; Greek capital letter Alpha */;
            case 914:
                return "&Beta;"     /* &#x0392; Greek capital letter Beta */;
            case 915:
                return "&Gamma;"    /* &#x0393; Greek capital letter Gamma */;
            case 916:
                return "&Delta;"    /* &#x0394; Greek capital letter Delta */;
            case 917:
                return "&Epsilon;"  /* &#x0395; Greek capital letter Epsilon */;
            case 918:
                return "&Zeta;"     /* &#x0396; Greek capital letter Zeta */;
            case 919:
                return "&Eta;"      /* &#x0397; Greek capital letter Eta */;
            case 920:
                return "&Theta;"    /* &#x0398; Greek capital letter Theta */;
            case 921:
                return "&Iota;"     /* &#x0399; Greek capital letter Iota */;
            case 922:
                return "&Kappa;"    /* &#x039a; Greek capital letter Kappa */;
            case 923:
                return "&Lambda;"   /* &#x039b; Greek capital letter Lambda */;
            case 924:
                return "&Mu;"       /* &#x039c; Greek capital letter Mu */;
            case 925:
                return "&Nu;"       /* &#x039d; Greek capital letter Nu */;
            case 926:
                return "&Xi;"       /* &#x039e; Greek capital letter Xi */;
            case 927:
                return "&Omicron;"  /* &#x039f; Greek capital letter Omicron */;
            case 928:
                return "&Pi;"       /* &#x03a0; Greek capital letter Pi */;
            case 929:
                return "&Rho;"      /* &#x03a1; Greek capital letter Rho */;
            case 931:
                return "&Sigma;"    /* &#x03a3; Greek capital letter Sigma */;
            case 932:
                return "&Tau;"      /* &#x03a4; Greek capital letter Tau */;
            case 933:
                return "&Upsilon;"  /* &#x03a5; Greek capital letter Upsilon */;
            case 934:
                return "&Phi;"      /* &#x03a6; Greek capital letter Phi */;
            case 935:
                return "&Chi;"      /* &#x03a7; Greek capital letter Chi */;
            case 936:
                return "&Psi;"      /* &#x03a8; Greek capital letter Psi */;
            case 937:
                return "&Omega;"    /* &#x03a9; Greek capital letter Omega */;
            case 945:
                return "&alpha;"    /* &#x03b1; Greek small letter alpha */;
            case 946:
                return "&beta;"     /* &#x03b2; Greek small letter beta */;
            case 947:
                return "&gamma;"    /* &#x03b3; Greek small letter gamma */;
            case 948:
                return "&delta;"    /* &#x03b4; Greek small letter delta */;
            case 949:
                return "&epsilon;"  /* &#x03b5; Greek small letter epsilon */;
            case 950:
                return "&zeta;"     /* &#x03b6; Greek small letter zeta */;
            case 951:
                return "&eta;"      /* &#x03b7; Greek small letter eta */;
            case 952:
                return "&theta;"    /* &#x03b8; Greek small letter theta */;
            case 953:
                return "&iota;"     /* &#x03b9; Greek small letter iota */;
            case 954:
                return "&kappa;"    /* &#x03ba; Greek small letter kappa */;
            case 955:
                return "&lambda;"   /* &#x03bb; Greek small letter lambda */;
            case 956:
                return "&mu;"       /* &#x03bc; Greek small letter mu */;
            case 957:
                return "&nu;"       /* &#x03bd; Greek small letter nu */;
            case 958:
                return "&xi;"       /* &#x03be; Greek small letter xi */;
            case 959:
                return "&omicron;"  /* &#x03bf; Greek small letter omicron */;
            case 960:
                return "&pi;"       /* &#x03c0; Greek small letter pi */;
            case 961:
                return "&rho;"      /* &#x03c1; Greek small letter rho */;
            case 962:
                return "&sigmaf;"   /* &#x03c2; Greek small letter final sigma */;
            case 963:
                return "&sigma;"    /* &#x03c3; Greek small letter sigma */;
            case 964:
                return "&tau;"      /* &#x03c4; Greek small letter tau */;
            case 965:
                return "&upsilon;"  /* &#x03c5; Greek small letter upsilon */;
            case 966:
                return "&phi;"      /* &#x03c6; Greek small letter phi */;
            case 967:
                return "&chi;"      /* &#x03c7; Greek small letter chi */;
            case 968:
                return "&psi;"      /* &#x03c8; Greek small letter psi */;
            case 969:
                return "&omega;"    /* &#x03c9; Greek small letter omega */;
            case 977:
                return "&thetasym;" /* &#x03d1; Greek theta symbol */;
            case 978:
                return "&upsih;"    /* &#x03d2; Greek upsilon with hook symbol */;
            case 982:
                return "&piv;"      /* &#x03d6; Greek pi symbol */;
            case 8194:
                return "&ensp;"     /* &#x2002; en space */;
            case 8195:
                return "&emsp;"     /* &#x2003; em space */;
            case 8201:
                return "&thinsp;"   /* &#x2009; thin space */;
            case 8204:
                return "&zwnj;"     /* &#x200c; zero width non-joiner */;
            case 8205:
                return "&zwj;"      /* &#x200d; zero width joiner */;
            case 8206:
                return "&lrm;"      /* &#x200e; left-to-right mark */;
            case 8207:
                return "&rlm;"      /* &#x200f; right-to-left mark */;
            case 8211:
                return "&ndash;"    /* &#x2013; en dash */;
            case 8212:
                return "&mdash;"    /* &#x2014; em dash */;
            case 8216:
                return "&lsquo;"    /* &#x2018; left single-6 quotation mark */;
            case 8217:
                return "&rsquo;"    /* &#x2019; right single-9 quotation mark */;
            case 8218:
                return "&sbquo;"    /* &#x201a; single low-9 quotation mark */;
            case 8220:
                return "&ldquo;"    /* &#x201c; left double-66 quotation mark */;
            case 8221:
                return "&rdquo;"    /* &#x201d; right double-99 quotation mark */;
            case 8222:
                return "&bdquo;"    /* &#x201e; double low-99 quotation mark */;
            case 8224:
                return "&dagger;"   /* &#x2020; dagger */;
            case 8225:
                return "&Dagger;"   /* &#x2021; double dagger */;
            case 8226:
                return "&bull;"     /* &#x2022; bullet */;
            case 8230:
                return "&hellip;"   /* &#x2026; horizontal ellipsis */;
            case 8240:
                return "&permil;"   /* &#x2030; per mille sign */;
            case 8242:
                return "&prime;"    /* &#x2032; prime */;
            case 8243:
                return "&Prime;"    /* &#x2033; double prime */;
            case 8249:
                return "&lsaquo;"   /* &#x2039; single left-pointing angle quotation mark */;
            case 8250:
                return "&rsaquo;"   /* &#x203a; single right-pointing angle quotation mark */;
            case 8254:
                return "&oline;"    /* &#x203e; overline */;
            case 8260:
                return "&frasl;"    /* &#x2044; fraction slash */;
            case 8364:
                return "&euro;"     /* &#x20ac; Euro currency sign */;
            case 8465:
                return "&image;"    /* &#x2111; black-letter capital i */;
            case 8472:
                return "&weierp;"   /* &#x2118; script capital p */;
            case 8476:
                return "&real;"     /* &#x211c; black-letter capital r */;
            case 8482:
                return "&trade;"    /* &#x2122; trademark sign */;
            case 8501:
                return "&alefsym;"  /* &#x2135; alef symbol */;
            case 8592:
                return "&larr;"     /* &#x2190; leftwards arrow */;
            case 8593:
                return "&uarr;"     /* &#x2191; upwards arrow */;
            case 8594:
                return "&rarr;"     /* &#x2192; rightwards arrow */;
            case 8595:
                return "&darr;"     /* &#x2193; downwards arrow */;
            case 8596:
                return "&harr;"     /* &#x2194; left right arrow */;
            case 8629:
                return "&crarr;"    /* &#x21b5; downwards arrow with corner leftwards */;
            case 8656:
                return "&lArr;"     /* &#x21d0; leftwards double arrow */;
            case 8657:
                return "&uArr;"     /* &#x21d1; upwards double arrow */;
            case 8658:
                return "&rArr;"     /* &#x21d2; rightwards double arrow */;
            case 8659:
                return "&dArr;"     /* &#x21d3; downwards double arrow */;
            case 8660:
                return "&hArr;"     /* &#x21d4; left right double arrow */;
            case 8704:
                return "&forall;"   /* &#x2200; for all */;
            case 8706:
                return "&part;"     /* &#x2202; partial differential */;
            case 8707:
                return "&exist;"    /* &#x2203; there exists */;
            case 8709:
                return "&empty;"    /* &#x2205; empty set */;
            case 8711:
                return "&nabla;"    /* &#x2207; nabla */;
            case 8712:
                return "&isin;"     /* &#x2208; element of */;
            case 8713:
                return "&notin;"    /* &#x2209; not an element of */;
            case 8715:
                return "&ni;"       /* &#x220b; like backwards epsilon */;
            case 8719:
                return "&prod;"     /* &#x220f; n-ary product */;
            case 8721:
                return "&sum;"      /* &#x2211; n-ary summation */;
            case 8722:
                return "&minus;"    /* &#x2212; minus sign */;
            case 8727:
                return "&lowast;"   /* &#x2217; asterisk operator */;
            case 8730:
                return "&radic;"    /* &#x221a; square root */;
            case 8733:
                return "&prop;"     /* &#x221d; proportional to */;
            case 8734:
                return "&infin;"    /* &#x221e; infinity */;
            case 8736:
                return "&ang;"      /* &#x2220; angle */;
            case 8743:
                return "&and;"      /* &#x2227; logical and */;
            case 8744:
                return "&or;"       /* &#x2228; vee */;
            case 8745:
                return "&cap;"      /* &#x2229; intersection */;
            case 8746:
                return "&cup;"      /* &#x222a; union */;
            case 8747:
                return "&int;"      /* &#x222b; integral */;
            case 8756:
                return "&there4;"   /* &#x2234; therefore three dots */;
            case 8764:
                return "&sim;"      /* &#x223c; tilde operator */;
            case 8773:
                return "&cong;"     /* &#x2245; congruent to */;
            case 8776:
                return "&asymp;"    /* &#x2248; asymptotic to */;
            case 8800:
                return "&ne;"       /* &#x2260; not equal to */;
            case 8801:
                return "&equiv;"    /* &#x2261; identical to */;
            case 8804:
                return "&le;"       /* &#x2264; less-than or equal to */;
            case 8805:
                return "&ge;"       /* &#x2265; greater-than or equal to */;
            case 8834:
                return "&sub;"      /* &#x2282; subset of */;
            case 8835:
                return "&sup;"      /* &#x2283; superset of */;
            case 8836:
                return "&nsub;"     /* &#x2284; not a subset of */;
            case 8838:
                return "&sube;"     /* &#x2286; subset of or equal to */;
            case 8839:
                return "&supe;"     /* &#x2287; superset of or equal to */;
            case 8853:
                return "&oplus;"    /* &#x2295; circled plus */;
            case 8855:
                return "&otimes;"   /* &#x2297; circled times */;
            case 8869:
                return "&perp;"     /* &#x22a5; up tack */;
            case 8901:
                return "&sdot;"     /* &#x22c5; dot operator */;
            case 8968:
                return "&lceil;"    /* &#x2308; left ceiling */;
            case 8969:
                return "&rceil;"    /* &#x2309; right ceiling */;
            case 8970:
                return "&lfloor;"   /* &#x230a; left floor */;
            case 8971:
                return "&rfloor;"   /* &#x230b; right floor */;
            case 9001:
                return "&lang;"     /* &#x2329; left-pointing angle bracket */;
            case 9002:
                return "&rang;"     /* &#x232a; right-pointing angle bracket */;
            case 9674:
                return "&loz;"      /* &#x25ca; open lozenge */;
            case 9824:
                return "&spades;"   /* &#x2660; black spade suit */;
            case 9827:
                return "&clubs;"    /* &#x2663; black club suit */;
            case 9829:
                return "&hearts;"   /* &#x2665; black heart suit */;
            case 9830:
                return "&diams;"    /* &#x2666; black diamond suit */;
            /* no named entities in range 0x2667 .. 0xffff */
            }// end switch
        // can't fall out bottom
        }// end charToEntity

    /**
     * convert a single char to its equivalent XML entity. Ordinary chars are not changed. 160 -> &nbsp;  weird chars
     * -> &#nnn; form
     *
     * @param c Char to convert
     *
     * @return equivalent string e.g. &amp;, null means leave char as is. Does not return unmodified letters.
     */
    private static String charToXMLEntity( char c )
        {
        switch ( c )
            {
            default:
                if ( c < 127 )
                    {
                    // leave alone as equivalent string.
                    return null;
                    // faster than String.valueOf( c ).intern();
                    }
                else
                    {
                    // use the &#nnn; form
                    return "&#" + Integer.toString( c ) + ";";
                    }
                // do NOT modify the following code. It is not generated.
            case 34:
                return "&quot;"/* &#x22; quotation mark */;
            case 38:
                return "&amp;"/* &#x26; ampersand */;
            // case 39:    // don't use apos, to make more compatible with HTML
            // return "&apos;"/* &#x27; apos */
            case 60:
                return "&lt;"/* &#x3c; less-than sign */;
            case 62:
                return "&gt;"/* &#x3e; greater-than sign */;
            case 160:
                return "&nbsp; /* &#x01; nbsp */;";
            }// end switch
        // can't fall out bottom
        }// end charToXMLEntity

    /**
     * Converts text to HTML by quoting dangerous characters. Text must not already contain entities. e.g. " ==> &quot;
     * < ==> &lt; ordinary text passes unchanged. Does not convert space to &nbsp;
     *
     * @param text     raw text to be processed. Must not be null.
     * @param xmlStyle true if insert basic 4 XML enities, otherwise use full HTML set.
     *
     * @return translated text, or null if input is null.
     * @noinspection WeakerAccess
     */
    private static String entifyHTMLorXML( String text, boolean xmlStyle )
        {
        if ( text == null )
            {
            return null;
            }
        int originalTextLength = text.length();
        // estimate text will grow by no more than 10%
        StringBuilder sb = new StringBuilder( originalTextLength * 110 / 100 );
        int charsToAppend = 0;
        for ( int i = 0; i < originalTextLength; i++ )
            {
            char c = text.charAt( i );
            String entity = xmlStyle ? charToXMLEntity( c ) : charToHTMLEntity( c );
            if ( entity == null )
                {
                // we could sb.append( c ), but that would be slower
                // than saving them up for a big append.
                charsToAppend++;
                }
            else
                {
                if ( charsToAppend != 0 )
                    {
                    sb.append( text.substring( i - charsToAppend, i ) );
                    charsToAppend = 0;
                    }
                sb.append( entity );
                }
            }// end for
        // append chars to the right of the last entity.
        if ( charsToAppend != 0 )
            {
            sb.append( text.substring( originalTextLength - charsToAppend,
                    originalTextLength ) );
            }
        // if result is not longer, we did not do anything. Save RAM by returning the original
        return ( sb.length() == originalTextLength ) ? text : sb.toString();
        }

    //    /** test hardness
    //     *
    //     * @param args  not used
    //     */
    //    public static void main ( String[] args )
    //    {
    //
    //    String s = "abc\"&xyz";
    //    System.out.println( entifyHTMLorXML( s , false  )) ;
    //    }
    }
